/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#include "PhilosopherAO.hpp"
#include "PhilosopherStateMachine.hpp"

int staticNumber = 0;
/*---------------------- PhilosopherAO:: active object -------------------------*/           

PhilosopherAO::PhilosopherAO( DWORD prio, Hsm* sm ) : AObject( prio, sm )
{
  n = staticNumber++;
}

/*---------------------- Root state of PhilosopherStateMachine:: -------------------------*/           
PhilosopherStateMachine::PhilosopherStateMachine() : 
  thinkingState( this ), hungryState( this ), eatingState( this )
{
  timer = ((PhilosopherAO*)getActiveObject())->getNumber() * 2937;
}

State*
PhilosopherStateMachine::fireInit()
{
  return &hungryState;
}

void
PhilosopherStateMachine::enter()
{
}

State*
PhilosopherStateMachine::fireEvent( Event *e )
{
  switch( e->eventId )
  {
    case tick :
      time++;
      break;
    default:
      break;
  }
  return 0; 
}

void
PhilosopherStateMachine::exit()
{
}

/*---------------------- State_Thinking -------------------------*/           

State*
State_Thinking::fireInit()
{
  return 0;
}

void
State_Thinking::enter()
{
  Event pe( sm->getActiveObject(), show );
  sm->getActiveObject()->publishEvent( &pe );
  sm->timer = (sm->time & 0x000001FF) * 2 + 270; // pseudo random number
}

State*
State_Thinking::fireEvent( Event *e )
{
  switch( e->eventId )
  {
    case tick :
      if( --(sm->timer) == 0 )
      {
        transition( &(sm->hungryState) );
      }
      break;
    default:
      break;
  }
  return getParent(); 
}

void
State_Thinking::exit()
{
}

/*---------------------- State_Hungry -------------------------*/           

State*
State_Hungry::fireInit()
{
  return 0;
}

void
State_Hungry::enter()
{
  Event pe( sm->getActiveObject(), show );
  sm->getActiveObject()->publishEvent( &pe );

  pe.eventId = hungry;
  sm->getActiveObject()->publishEvent( &pe );
}

State*
State_Hungry::fireEvent( Event *e )
{
  switch( e->eventId )
  {
    case tick :
      break;
    default:
      if( e->eventId == (eating + ((PhilosopherAO*)sm->getActiveObject())->getNumber()) )
      {
        transition( &(sm->eatingState) );
      }
      break;
  }
  return getParent(); 
}

void
State_Hungry::exit()
{
}
/*---------------------- State_Eating -------------------------*/           

State*
State_Eating::fireInit()
{
  return 0;
}

void
State_Eating::enter()
{
  Event pe( sm->getActiveObject(), show );
  sm->getActiveObject()->publishEvent( &pe );
  sm->timer = (sm->time & 0x000001FF) + 270; // pseudo random number
}

State*
State_Eating::fireEvent( Event *e )
{
  switch( e->eventId )
  {
    case tick :
      if( --(sm->timer) == 0 )
      {
        transition( &(sm->thinkingState) );
      }
      break;
    default:
      break;
  }
  return getParent(); 
}

void
State_Eating::exit()
{
  Event pe( sm->getActiveObject(), done );
  sm->getActiveObject()->publishEvent( &pe );
}
