/**
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#include "Timer.hpp"
#include "pc.hpp" // for debug only

/** creates IS Active object with priority p and interrupt vector 0 */
Timer::Timer( DWORD p ) : ISAObject( p, 0 )
{
  second = 100;
  period = 0;
}

AO_STACK *
Timer::serviceInterrupt( AO_STACK * )
{
  /* 
   * We are inside Interrupt Service Routine.
   * In this point Timer can do the all job that need immediate processing.
   * But if event processing can be delayed Timer send the event to itself and
   * own thread will process it by processEvent() method that is running with own priority.
   */ 
  Event e( this, tick );
  receiveEvent( &e );
}

DWORD
Timer::processEvent( Event * event )
{
  publishEvent( event );
  char p;
  if( --second < 0 )
  {
//    publishEvent( event );
    second = 100;
    switch( period++ )
    {
      case 0:
        p = '-';
        break;
      case 1:
        p = '\\';
        break;
      case 2:
        p = '|';
        break;
      case 3:
        p = '/';
        break;
    }
    debugPrint( 72, "TIMER\0" );
    debugPrint( 78, p );
    if( period > 3)
      period = 0;
  }
  return 1;
}
