;--------------------------------------------------------------------------
;   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)
;
;   Licensed under the Apache License, Version 2.0 (the "License");
;   you may not use this file except in compliance with the License.
;   You may obtain a copy of the License at
;
;       http://www.apache.org/licenses/LICENSE-2.0
;
;   Unless required by applicable law or agreed to in writing, software
;   distributed under the License is distributed on an "AS IS" BASIS,
;   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
;   See the License for the specific language governing permissions and
;   limitations under the License. 
;--------------------------------------------------------------------------

;********************************************************************************************************
;                   Intel 80x86 (Protected-Mode, Flat Model)
;                         INTERRUPT SERVICE ROUTINES:
;********************************************************************************************************

; macro for mask of interrupt controller 8259
%macro intc_mask_int 1
%if (%1 >= 0x20) && (%1 < 0x28)
%assign port 0x21
%else
%assign port 0xA1
%endif
               mov     ah, (1 << ( %1 & 0x07 ))
               in      al, port
               or      al, ah
               out     port, al
%endmacro

; macro for unmask of interrupt controller 8259
%macro intc_unmask_int 1
%if (%1 >= 0x20) && (%1 <= 0x28)
%assign port 0x21
%else
%assign port 0xA1
%endif
               mov     ah, ~(1 << ( %1 & 0x07 ))
               in      al, port
               and     al, ah
               out     port, al
%endmacro

; my macro for acknoledge of interrupt controller
%macro intc_ack 1
               mov      al, 0x20       ; non-specific EOI
%if (%1 >= 0x20) && (%1 <= 0x28)
               out      0x20, al       ;
%else
               out      0xA0, al       ;
%endif
%endmacro

;********************************************************************************************************
;                                    PUBLIC and EXTERNAL REFERENCES
;********************************************************************************************************

            global timerISR
            global schedulerISR
            global __gxx_personality_v0
            global free

            extern  processInterrupt        ; AO_STACK * ISAObject::processInterrupt( DWORD, AO_STACK * )t

segment     .text
__gxx_personality_v0:   dd     0
free:                   dd     0 
;*********************************************************************************************************
;                                            HANDLE TICK ISR
;
;*********************************************************************************************************
timerISR:
               pushad                         ; Save interrupted thread's context

               push    esp                     ; the parameter of processInterrupt( TH_STACK * stkp )
               push    dword 0                 ; the parameter of processInterrupt( int )
               call    processInterrupt 
               mov     esp, eax     ; gets a return from the routine and puts it to esp -> switch context of CPU
               intc_ack 0x20                  ; serves 8259 PIC
               popad    
               iret                            ; Return to interrupted thread

;*********************************************************************************************************
;                                       SCHEDULER'S PROGRAM INTERRUPT SR
;
;*********************************************************************************************************
schedulerISR:
               pushad   
               push    esp                     ; the parameter of processInterrupt( TH_STACK * stkp )
               push    dword 16                ; the parameter of processInterrupt( int )
               call    processInterrupt 
               mov     esp, eax     ; gets a return from the routine and puts it to esp -> switch context of CPU
               popad    
               iret                            ; Return to interrupted thread

;*********************************************************************************************************
;                                            KEYBORD ISR
;
;*********************************************************************************************************
keyboardISR:
               pushad                         ; Save interrupted thread's context

               push    esp                     ; the parameter of processInterrupt( TH_STACK * stkp )
               push    dword 1                 ; the parameter of processInterrupt( int )
               call    processInterrupt 
               mov     esp, eax     ; gets a return from the routine and puts it to esp -> switch context of CPU
               intc_ack 0x21                  ; serves 8259 PIC
               popad    
               iret                            ; Return to interrupted thread
               