/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#include "pc.hpp"

void
Display::clearScreen( int ch )
{ BYTE  *pscr = (BYTE *)(baseAddress);
  
  for( int i = 0; i < (maxX * maxY); i++)
  {                                                     // PC display has 80 columns and 25 lines        
    *pscr++ = (BYTE)ch;                                // Put character in video RAM                
    *pscr++ = (BYTE)color;                             // Put video attribute in video RAM              
  }
}

void  
Display::clearColomn( int ch, int x )
{ BYTE *pscr = (BYTE *)(baseAddress + x * 2);
  for( int i = 0; i < maxY; i++) 
  {
    *pscr++ = (BYTE)ch;             // Put character in video RAM                     
    *pscr-- = (BYTE)color;          // Put video attribute in video RAM                   
     pscr = pscr + maxX * 2;         // Position on next row                               
  }
}

void
Display::clearRow( int ch, int y )
{ BYTE *pscr= (BYTE *)(baseAddress + y * maxX * 2);

  for( int i = 0; i < maxX; i++) 
  {
    *pscr++ = (BYTE)ch;               // Put character in video RAM                    
    *pscr++ = (BYTE)color;            // Put video attribute in video RAM                   
  }
}

void  
Display::print( char s )
{ BYTE  *pscr = (BYTE *)(baseAddress + y * maxX * 2 + x * 2);

  *pscr++ = s;                         // Put character in video RAM               
  *pscr   = (BYTE)color;              // Put video attribute in video RAM 
  if( ++x >= maxX )
  {
    x = 0;
    y++;
  };        
}

void
Display::print( char * s )
{
  while( (*s) != 0 )
  { 
    print( *s++ );
  } 
}

void  
Display::printf( char * format, ... )
{
  char out[250];
  va_list ap;
  va_start( ap, format );
  FormatParser fp( out, ap );
  fp.init();
  
  while( !fp.isFinish() && !fp.isError() )
  {
    fp.dispatch( *format++ );
  }
  va_end( ap );
  print( out );
}

void  
Display::sprintf( char * out, char * format, ... )
{
  va_list ap;
  va_start( ap, format );
  FormatParser fp( out, ap );
  fp.init();
  
  while( !fp.isFinish() && !fp.isError() )
  {
    fp.dispatch( *format++ );
  }
  va_end( ap );
}

void     
FormatParser::initial( char )
{
  output[0] = 0;
  s = output;
  align = 0; 
  width = 0;
  TRAN( &FormatParser::text );
}

void     
FormatParser::text( char sig )
{
  switch( sig )
  {
    case '%' :
      width = 0;
      align = 0;
      TRAN( &FormatParser::control );
      break;
    case '\0' :
      *s = sig;
      TRAN( &FormatParser::finish );
      break;
    default :
      *s++ = sig;
  }
}

void     
FormatParser::control( char sig )
{
  switch( sig )
  {
    case '%' :
      *s++ = sig;
      TRAN( &FormatParser::text );
      break;
    case '-' :
      align = 1;
      break;
    case '0' : case '1' : case '2' : case '3' : case '4' : 
    case '5' : case '6' : case '7' : case '8' : case '9' :
      width = (int) (sig - '0');
      TRAN( &FormatParser::controlDig );
      break;
    case 's' : case 'd' : case 'h' : case 'u' :
      processParam( sig );
      break;
    default :
      *s = 0;
      TRAN( &FormatParser::error );
  }
}

void     
FormatParser::controlDig( char sig )
{
  switch( sig )
  {
    case '0' : case '1' : case '2' : case '3' : case '4' : 
    case '5' : case '6' : case '7' : case '8' : case '9' :
      width = width * 10 + (int)(sig - '0');
      break;
    case 's' : case 'd' : case 'h' : case 'u' :
      processParam( sig );
      break;
    default :
      *s = 0;
      TRAN( &FormatParser::error );
  }
}

void     
FormatParser::controlCmd( char sig )
{
  switch( sig )
  {
    case '%' :
      width = 0;
      align = 0;
      TRAN( &FormatParser::control );
      break;
    case '\0' :
      *s = 0;
      TRAN( &FormatParser::error );
      break;
    default :
      *s++ = sig;
      TRAN( &FormatParser::text );
  }
}

void     
FormatParser::processParam( char sig )
{
  if( align == 1 )
    width = -width;
  if( sig != 's' )
  {
    s = intToString( s, va_arg( valist, int ), width, sig );
  }
  else
    if( sig == 's' ) 
    {
      s = stringToString( s, va_arg( valist, char*), width );
    }
    else
    {
      *s = 0;
      TRAN( &FormatParser::error );
      return;
    } 
  TRAN( &FormatParser::controlCmd );
}

void     
FormatParser::finish( char )
{
}

void     
FormatParser::error( char )
{
}

char *
stringToString( char *s, char * arg, int width )
{
  int i = 0, align = 0, length, j;
  char *e = arg;

  if( width < 0 )
  {
    width = -width;
    align = 1;  
  }
                
  while( *e++ != 0 )
  {
    i++;
  };
  length = i;
  if( width == 0 )
  {  
    width = length;
  }
  else  
  { 
    if( length > width )
    {
      i = 0;
      while( width > i++ )
        *(s++) = '*';
      return s;
    }
  }
  e = arg;
  i = 0;
  if( align == 0 )
  {
    for( ; i < width - length; i++ )
      *(s + i) = ' ';
  }    
  for( j = 0; j < length; j++,i++ )
    *(s + i) = *(e++);
  for( ; i < width; i++ )
    *(s + i) = ' ';
  return (s + width);
}

char *
intToString( char *s, int arg, int width, char type )
{
  static char digits[] = {'0','1','2','3','4','5','6','7','8','9',
                          'A','B','C','D','E','F'};
  unsigned int a;
  char buf[80];
  int i, j, length, align = 0, minus = 0, base = ( type == 'h')? 16 : 10;
  if( width < 0 )
  {
    width = -width;
    align = 1;  
  }
  if( type == 'u' || type == 'h' )
    a = arg;
  else
  { 
    if( arg < 0 )
    {
      a = -arg;
      minus = 1; 
    }
    else
      a = arg;
  }    
  i = 0;
  if( a == 0 )
  {
    *(buf + i++) = digits[0];
  }
  else
    while( a > 0 )
    {
      *(buf + i++) = digits[a % base];
      a = a / base;
    }
  if( minus == 1 )
  {
    *(buf + i++) = '-';
  }
  length = i;
  if( width == 0 )
    width = length;
  else  
  { 
    if( length > width )
    {
      i = 0;
      while( width > i )
        *(buf + i++) = '*';
      length = width;  
    }
  }
  i = 0;
  if( align == 0 )
  {
    for( ; i < width - length; i++ )
      *(s + i) = ' ';
  }    
  for( j = 0; j < length; i++, j++ )
    *(s + i) = *(buf + length - j - 1);
  for( ; i < width; i++ )
    *(s + i) = ' ';
  return (s + width);
};

/*
*********************************************************************************************************
*                                        OBTAIN INTERRUPT VECTOR
* Description: This function reads the pointer stored at the specified vector.
* Arguments  : vect  is the desired interrupt vector number, a number between 0 and 255.
* Returns    : The address of the Interrupt handler stored at the desired vector location.
*********************************************************************************************************
*/
void * PC_VectGet( BYTE vect )
{
    WORD    *pvect;
    DWORD     hi_isr;
    DWORD     lo_isr;

    pvect = (WORD *)(idtStart + vect * 8);              /* Point into IDT at desired vector location     */
//    OS_ENTER_CRITICAL();
    hi_isr   = *(pvect + 3);                            /* Obtain the vector's OFFSET                    */
    lo_isr   = *pvect;
//    OS_EXIT_CRITICAL();
    return (void *)((hi_isr << 16) | (lo_isr & 0xFFFF));
}

/*
*********************************************************************************************************
*                                        INSTALL INTERRUPT VECTOR
* Description: This function sets an interrupt vector in the interrupt vector table.
* Arguments  : vect  is the desired interrupt vector number, a number between 0 and 255.
*              isr   is a pointer to a function to execute when the interrupt or exception occurs.
* Returns    : none
*********************************************************************************************************
*/
void PC_VectSet( BYTE vect, void (*isr)(void) )
{
    WORD    *pvect;
 
    pvect    = (WORD *)(idtStart + vect * 8);     /* Point into IDT at desired vector location     */
//    OS_ENTER_CRITICAL();
    *(pvect + 3) = (WORD)((DWORD)isr >> 16);                  /* Store ISR offset*/
    *pvect   = (WORD)((DWORD)isr & 0x0000FFFF);
//    OS_EXIT_CRITICAL();
}

void  
debugPrint( int pos, char s )
{ BYTE  *pscr = (BYTE *)(0xb8000 + pos * 2);

  *pscr++ = s;                         // Put character in video RAM
  *pscr = 0x07;               
}

void
debugPrint( int pos, char* s )
{
  while( *s != 0 )
  {
    debugPrint( pos++, *s );
    s++;
  }
}
